@@ -33,13 +33,17 @@ namespace Beagle.Filters {
 
 	[PropertyKeywordMapping (Keyword="album", PropertyName="fixme:album", IsKeyword=false)]
 	[PropertyKeywordMapping (Keyword="artist", PropertyName="fixme:artist", IsKeyword=false)]
+	[PropertyKeywordMapping (Keyword="genre", PropertyName="fixme:genre", IsKeyword=true)]
 	public class FilterMusic : Beagle.Daemon.Filter {
 	
 		public FilterMusic ()
 		{
 			// 1: Added duration and bitrate property
 			SetVersion (1);
-			
+		}
+
+		protected override void RegisterSupportedTypes ()
+		{
 			// APE / Monkeys Audio
 			AddSupportedFlavor (FilterFlavor.NewFromExtension (".ape"));
 
