@@ -40,10 +40,29 @@ namespace Beagle.Filters {
     
 	public class FilterOpenOffice : Beagle.Daemon.Filter {
 
-		Hashtable hotStyles;
-		bool odtFormat;
-		
+		private Hashtable hotStyles;
+
+		private Hashtable attr_to_index = null;
+		private Hashtable InterestingAttribute {
+			get {
+				if (attr_to_index != null)
+					return attr_to_index;
+
+				attr_to_index = new Hashtable (1 /* Number of attributes */);
+				// Add interesting node-attribute pairs to this hashtable
+				// attr_to_index [node_name] = attribute_name
+				attr_to_index ["table:table"] = "table:name";
+
+				return attr_to_index;
+			}
+		}
+
 		public FilterOpenOffice () 
+		{
+			SnippetMode = true;
+		}
+
+		protected override void RegisterSupportedTypes ()
 		{
 			// OO 1.0 mime types
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.sun.xml.writer"));
@@ -64,9 +83,6 @@ namespace Beagle.Filters {
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.oasis.opendocument.presentation-template"));
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.oasis.opendocument.graphics"));
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.oasis.opendocument.graphics-template"));
-
-			SnippetMode = true;
-			odtFormat = false;
 		}
 		
 		// Parse the "style" nodes and mark appropriate styles as *HOT*
@@ -139,12 +155,14 @@ namespace Beagle.Filters {
 			return nodeName == "text:footnote"
 				|| nodeName == "text:endnote"
 				|| nodeName == "office:annotation"
+				|| nodeName == "table:table"
 				|| nodeName == "text:note";       // "ODT format"
 		}
 
 		static bool NodeBreaksTextAfter (String nodeName)
 		{
-			return  nodeName == "text:s"
+			return  nodeName == "text:line-break"
+				|| nodeName == "text:s"
 				|| nodeName == "text:tab-stop"
 				|| nodeName == "table:table-cell";
 		}
@@ -160,7 +178,7 @@ namespace Beagle.Filters {
 		}
 
 		private Stack hot_nodes = new Stack ();
-		private string strPartText = "";
+		private string strPartText = String.Empty;
 		private bool bPartHotStyle = false;
 		private Stack hot_container_nodes = new Stack ();
 
@@ -201,8 +219,8 @@ namespace Beagle.Filters {
 			} else {
 				strTemp = strPartText + paramStr;
 				strPartText = strTemp;
-				paramStr = "";
-				strTemp = "";
+				paramStr = String.Empty;
+				strTemp = String.Empty;
 			}
 					
 			// Enable *HOT* just before appending the text
@@ -215,13 +233,21 @@ namespace Beagle.Filters {
 			} else 
 				bPartHotStyle |= false;
 
-			if (paramStr.Length > 0)
-				AppendText (paramStr);
+			AppendText (paramStr);
 
 			if (strPartText.Trim().Length < 1)
 				bPartHotStyle = false;
 		}
 
+		void IndexAttribute (string node_name, string attr_name, string attr_value)
+		{
+			if (attr_value == null || attr_value.Length == 0)
+				return;
+
+			if ((string)InterestingAttribute [node_name] == attr_name)
+				AppendText (attr_value);	
+		}
+
 		void FlushPartialText () 
 		{
 			if (strPartText.Length > 0) {
@@ -230,7 +256,7 @@ namespace Beagle.Filters {
 				AppendText (strPartText);
 				if (IsHot)
 					HotDown ();
-				strPartText = "";
+				strPartText = String.Empty;
 			}
 			bPartHotStyle = false;
 
@@ -259,7 +285,10 @@ namespace Beagle.Filters {
 						continue;
 					}
 
-					if (reader.Name == "style:style") {
+					// FIXME: Allow adding more style nodes
+					if (reader.Name == "style:style"
+					    || reader.Name == "number:date-style"
+					    || reader.Name == "style:font-decl") {
 						StudyStyleNode (reader); 
 						continue;
 					}
@@ -281,13 +310,16 @@ namespace Beagle.Filters {
 						AppendWhiteSpace ();
 						isHot = false;
 					} else {
+						string node_name = reader.Name;
 						bool has_attr = reader.MoveToFirstAttribute ();
 						while (has_attr) {
 							if (reader.Name.EndsWith(":style-name")) {
 								if (hotStyles.Contains (reader.Value))
 									isHot = true;
 								break;
-							}
+							} else
+								IndexAttribute (node_name, reader.Name, reader.Value);
+
 							has_attr = reader.MoveToNextAttribute ();
 						}
 						reader.MoveToElement();
@@ -389,8 +421,8 @@ namespace Beagle.Filters {
 				}
 				reader.Read ();
 			}
-			if (slideCount != null)
-				AddProperty (Beagle.Property.NewUnsearched ("fixme:slide-count", slideCount));
+
+			AddProperty (Beagle.Property.NewUnsearched ("fixme:slide-count", slideCount));
 		}
 
 		private void ExtractMetadata (XmlReader reader)
@@ -427,11 +459,9 @@ namespace Beagle.Filters {
 					
 				case "meta:document-statistic":
 					string attr = reader.GetAttribute ("meta:page-count");
-					if (attr != null)
-						AddProperty (Beagle.Property.NewUnsearched ("fixme:page-count", attr));
+					AddProperty (Beagle.Property.NewUnsearched ("fixme:page-count", attr));
 					attr = reader.GetAttribute ("meta:word-count");
-					if (attr != null)
-						AddProperty (Beagle.Property.NewUnsearched ("fixme:word-count", attr));
+					AddProperty (Beagle.Property.NewUnsearched ("fixme:word-count", attr));
 
 					// Both writer and calc uses this attribute.  writer stores the
 					// count of tables in a sxw whereas calc stores the count of
@@ -446,10 +476,8 @@ namespace Beagle.Filters {
 					string name = reader.GetAttribute ("meta:name");
 					reader.Read ();
 
-					if (reader.Value != "") {
-						AddProperty (Beagle.Property.New ("fixme:UserDefined-" + name,
+					AddProperty (Beagle.Property.New ("fixme:UserDefined-" + name,
 										  reader.Value));
-					}
 					break;
 
 				case "meta:keyword":
@@ -471,10 +499,10 @@ namespace Beagle.Filters {
 			try {
 				zip = new ZipFile (info.FullName);
 
-				if (MimeType.StartsWith ("application/vnd.oasis.opendocument."))
-					odtFormat = true;
+				//if (MimeType.StartsWith ("application/vnd.oasis.opendocument."))
+				//	odtFormat = true;
 
-			} catch (Exception e) {
+			} catch (Exception) {
 				Logger.Log.Error ("Unable to open {0}.  Probably an invalid OpenOffice document.", 
 						  info.FullName);
 				Finished ();
@@ -536,5 +564,11 @@ namespace Beagle.Filters {
 			if ((WalkContentNodes (style_reader)) && (WalkContentNodes (content_reader)))
 				Finished ();
 		}
+
+		override protected void DoClose ()
+		{
+			if (zip != null)
+				zip.Close ();
+		}
 	}
 }
