@@ -146,15 +146,16 @@ namespace Beagle.Daemon {
 			return impls.Length > 0;
 		}
 
-		// For every type in the assembly that
-		// (1) implements IQueryable
-		// (2) Has a QueryableFlavor attribute attached
+		// Find the types in the assembly that
+		// (1) register themselves in AssemblyInfo.cs:IQueryableTypes and
+		// (2) has a QueryableFlavor attribute attached
 		// assemble a Queryable object and stick it into our list of queryables.
-		static void ScanAssembly (Assembly assembly)
+		static void ScanAssemblyForQueryables (Assembly assembly)
 		{
 			int count = 0;
 
-			foreach (Type type in ReflectionFu.ScanAssemblyForInterface (assembly, typeof (IQueryable))) {
+			foreach (Type type in ReflectionFu.GetTypesFromAssemblyAttribute (assembly, typeof (IQueryableTypesAttribute))) {
+				bool type_accepted = false;
 				foreach (QueryableFlavor flavor in ReflectionFu.ScanTypeForAttribute (type, typeof (QueryableFlavor))) {
 					if (! UseQueryable (flavor.Name))
 						continue;
@@ -173,8 +174,7 @@ namespace Beagle.Daemon {
 					try {
 						iq = Activator.CreateInstance (type) as IQueryable;
 					} catch (Exception e) {
-						Logger.Log.Error ("Caught exception while instantiating {0} backend", flavor.Name);
-						Logger.Log.Error (e);
+						Logger.Log.Error (e, "Caught exception while instantiating {0} backend", flavor.Name);
 					}
 
 					if (iq != null) {
@@ -182,15 +182,59 @@ namespace Beagle.Daemon {
 						queryables.Add (q);
 						iqueryable_to_queryable [iq] = q;
 						++count;
+						type_accepted = true;
 						break;
 					}
 				}
+
+				if (! type_accepted)
+					continue;
+
+				object[] attributes = type.GetCustomAttributes (false);
+				foreach (object attribute in attributes) {
+					PropertyKeywordMapping mapping = attribute as PropertyKeywordMapping;
+					if (mapping == null)
+						continue;
+					//Logger.Log.Debug (mapping.Keyword + " => " 
+					//		+ mapping.PropertyName + 
+					//		+ " is-keyword=" + mapping.IsKeyword + " (" 
+					//		+ mapping.Description + ") "
+					//		+ "(" + type.FullName + ")");
+					PropertyKeywordFu.RegisterMapping (mapping);
+				}
+					
 			}
 			Logger.Log.Debug ("Found {0} backends in {1}", count, assembly.Location);
 		}
 
 		////////////////////////////////////////////////////////
 
+		public static void ReadKeywordMappings ()
+		{
+			Logger.Log.Debug ("Reading mapping from filters");
+			ArrayList assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_FILTER_PATH", PathFinder.FilterDir);
+
+			foreach (Assembly assembly in assemblies) {
+				foreach (Type type in ReflectionFu.GetTypesFromAssemblyAttribute (assembly, typeof (FilterTypesAttribute))) {
+					object[] attributes = type.GetCustomAttributes (false);
+					foreach (object attribute in attributes) {
+						
+						PropertyKeywordMapping mapping = attribute as PropertyKeywordMapping;
+						if (mapping == null)
+							continue;
+						//Logger.Log.Debug (mapping.Keyword + " => " 
+						//		+ mapping.PropertyName
+						//		+ " is-keyword=" + mapping.IsKeyword + " (" 
+						//		+ mapping.Description + ") "
+						//		+ "(" + type.FullName + ")");
+						PropertyKeywordFu.RegisterMapping (mapping);
+					}
+				}
+			}
+		}
+
+		////////////////////////////////////////////////////////
+
 		// Scans PathFinder.SystemIndexesDir after available 
 		// system-wide indexes.
 		static void LoadSystemIndexes () 
@@ -253,8 +297,7 @@ namespace Beagle.Daemon {
 				Logger.Log.Warn ("Unable to create read-only index (likely due to index version mismatch): {0}", index_dir.FullName);
 				return false;
 			} catch (Exception e) {
-				Logger.Log.Error ("Caught exception while instantiating static queryable: {0}", index_dir.Name);
-				Logger.Log.Error (e);					
+				Logger.Log.Error (e, "Caught exception while instantiating static queryable: {0}", index_dir.Name);
 				return false;
 			}
 			
@@ -276,23 +319,35 @@ namespace Beagle.Daemon {
 
 		////////////////////////////////////////////////////////
 
-		static public void Start ()
+		private static ArrayList assemblies = null;
+
+		// Perform expensive initialization steps all at once.
+		// Should be done before SignalHandler comes into play.
+		static public void Init ()
 		{
 			ReadBackendsFromConf ();
+			assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_BACKEND_PATH", PathFinder.BackendDir);
+		}
 
-			ArrayList assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_BACKEND_PATH", PathFinder.BackendDir);
+		private static bool queryables_started = false;
 
+		static public void Start ()
+		{
 			// Only add the executing assembly if we haven't already loaded it.
 			if (assemblies.IndexOf (Assembly.GetExecutingAssembly ()) == -1)
 				assemblies.Add (Assembly.GetExecutingAssembly ());
 
 			foreach (Assembly assembly in assemblies) {
-				ScanAssembly (assembly);
+				ScanAssemblyForQueryables (assembly);
 
 				// This allows backends to define their
 				// own executors.
 				Server.ScanAssemblyForExecutors (assembly);
 			}
+			
+			assemblies = null;
+
+			ReadKeywordMappings ();
 
 			LoadSystemIndexes ();
 			LoadStaticQueryables ();
@@ -314,6 +369,8 @@ namespace Beagle.Daemon {
 				q.Start ();
 			}
 
+			queryables_started = true;
+
 			return false;
 		}
 
@@ -328,7 +385,7 @@ namespace Beagle.Daemon {
 			string ret = "User:\n";
 
 			foreach (Assembly assembly in assemblies) {
-				foreach (Type type in ReflectionFu.ScanAssemblyForInterface (assembly, typeof (IQueryable))) {
+				foreach (Type type in ReflectionFu.GetTypesFromAssemblyAttribute (assembly, typeof (IQueryableTypesAttribute))) {
 					foreach (QueryableFlavor flavor in ReflectionFu.ScanTypeForAttribute (type, typeof (QueryableFlavor)))
 						ret += String.Format (" - {0}\n", flavor.Name);
 				}
@@ -558,35 +615,34 @@ namespace Beagle.Daemon {
 
 		////////////////////////////////////////////////////////
 
-		static public string GetIndexInformation ()
+		static public IEnumerable GetIndexInformation ()
 		{
-			StringBuilder builder = new StringBuilder ('\n');
-
-			foreach (Queryable q in queryables) {
-				QueryableStatus status = q.GetQueryableStatus ();
-
-				builder.Append ("Name: ").Append (status.Name).Append ('\n');
-				builder.Append ("Count: ").Append (status.ItemCount).Append ('\n');
-				builder.Append ("Indexing: ").Append (status.IsIndexing).Append ('\n');
-				builder.Append ('\n');
-			}
-
-			return builder.ToString ();
+			foreach (Queryable q in queryables)
+				yield return q.GetQueryableStatus ();
 		}
 
 		////////////////////////////////////////////////////////
 
 		static public bool IsIndexing {
 			get {
+				// If the backends haven't been started yet,
+				// there is at least the initial setup.  Just
+				// assume all the backends are indexing.
+				if (! queryables_started)
+					return true;
+
 				foreach (Queryable q in queryables) {
 					QueryableStatus status = q.GetQueryableStatus ();
 
+					if (status == null)
+						return false;
+
 					if (status.IsIndexing)
 						return true;
 				}
 
 				return false;
 			}
-		}					
+		}
 	}
 }
