@@ -23,11 +23,11 @@ namespace Search.Tiles {
 
 		public TileFile (Beagle.Hit hit, Beagle.Query query) : base (hit, query)
 		{
-			Title = GetTitle ();
+			Title = GetTitle (hit);
 			EnableOpenWith = true;
 			
 			if (Hit.FileInfo != null) {
-				Timestamp = Hit.FileInfo.LastWriteTime;
+				Timestamp = Hit.FileInfo.LastWriteTimeUtc;
 				Description = Utils.NiceShortDate (Timestamp);
 			}
 
@@ -41,20 +41,36 @@ namespace Search.Tiles {
 
 		protected override void LoadIcon (Gtk.Image image, int size)
 		{
+			// The File tile doesn't respect the icon size because
+			// 48 is too small for thumbnails
 			if (!thumbnailer.SetThumbnailIcon (image, Hit, size))
 				base.LoadIcon (image, size);
 		}
 
-		private string GetTitle ()
+		protected static string GetTitle (Beagle.Hit hit, bool get_parent)
 		{
-			string title = Hit.GetFirstProperty ("dc:title");
+			string title;
 
-			if (title == null || title == "")
-				title = Hit.GetFirstProperty ("beagle:ExactFilename");
+			if (get_parent)
+				title = Utils.GetFirstPropertyOfParent (hit, "dc:title");
+			else
+				title = hit.GetFirstProperty ("dc:title");
+
+			if (title == null || title == "") {
+				if (get_parent)
+					title = Utils.GetFirstPropertyOfParent (hit, "beagle:ExactFilename");
+				else
+					title = hit.GetFirstProperty ("beagle:ExactFilename");
+			}
 
 			return title;
 		}
 
+		protected static string GetTitle (Beagle.Hit hit)
+		{
+			return GetTitle (hit, false);
+		}
+
 		public override void Open ()
 		{
 			base.OpenFromMime (Hit);
@@ -76,6 +92,8 @@ namespace Search.Tiles {
 
 #if ENABLE_DESKTOP_LAUNCH
 			p.Arguments = new string [] { "desktop-launch", path };
+#elif ENABLE_XDG_OPEN
+			p.Arguments = new string [] { "xdg-open", path };
 #else
 			p.Arguments = new string [] { "nautilus", "--no-desktop", path };
 #endif
@@ -88,13 +106,10 @@ namespace Search.Tiles {
 
 		public void Email ()
 		{
-			SafeProcess p = new SafeProcess ();
-			p.Arguments = new string [] { "evolution", String.Format ("mailto:?attach={0}", Hit.FileInfo.FullName) };
-
 			try {
-				p.Start () ;
+				OpenFromUri (String.Format ("mailto:?attach={0}", Hit.FileInfo.FullName));
 			} catch (Exception e) {
-				Console.WriteLine ("Error launching Evolution composer: " + e);
+				Console.WriteLine ("Error sending email: " + e);
 			}
 		}
 
@@ -122,21 +137,14 @@ namespace Search.Tiles {
 		{
 			DetailsPane details = new DetailsPane ();
 
-			details.AddLabelPair (Catalog.GetString ("Title:"),
-					      GetTitle (),
-					      0, 1);
-			details.AddLabelPair (Catalog.GetString ("Last Edited:"),
-					      Utils.NiceLongDate (Timestamp),
-					      1, 1);
-			if (Hit ["dc:author"] != null) {
-				details.AddLabelPair (Catalog.GetString ("Author:"),
-						      Hit ["dc:author"],
-						      1, 3);
-			}
-			details.AddLabelPair (Catalog.GetString ("Full Path:"),
-					      Hit.Uri.LocalPath,
-					      2, 1);
-			details.AddSnippet (3, 1);
+			details.AddLabelPair (Catalog.GetString ("Title:"), GetTitle (Hit));
+			details.AddLabelPair (Catalog.GetString ("Last Edited:"), Utils.NiceLongDate (Timestamp));
+
+			if (Hit ["dc:author"] != null)
+				details.AddLabelPair (Catalog.GetString ("Author:"), Hit ["dc:author"]);
+
+			details.AddLabelPair (Catalog.GetString ("Full Path:"), Hit.Uri.LocalPath);
+			details.AddSnippet ();
 
 			return details;
 		}
